@@ -113,8 +113,18 @@ case "x$type" in
     x ) type=test ;;
 esac
 case "x$algo" in
-    xrsa ) algo=RSA ;;
-    xdsa ) algo=DSA ;;
+    xRSA|xrsa ) 
+        algo=RSA
+        ;;
+    xDSA|xdsa ) 
+        algo=DSA 
+        ;;
+    x ) 
+        algo=choose
+        ;;
+    * ) echo "Unknown algorithm \'$algo' (use RSA or DSA!)" 1>&2
+        exit 1
+        ;;
 esac
 
 #   processing
@@ -125,7 +135,7 @@ case $type in
         echo "${T_MD}Generating self-signed Snake Oil certificate [DUMMY]${T_ME}"
         echo "______________________________________________________________________"
         echo ""
-        if [ ".$algo" = . ]; then
+        if [ ".$algo" = .choose ]; then
             algo=RSA
         fi
         if [ ".$algo" = .RSA ]; then
@@ -135,6 +145,7 @@ case $type in
             cp $sslcrtdir/snakeoil-dsa.crt $sslcrtdir/server.crt
             cp $sslkeydir/snakeoil-dsa.key $sslkeydir/server.key
         fi
+        chmod 600 $sslkeydir/server.key
         echo "${T_MD}RESULT: Server Certification Files${T_ME}"
         echo ""
         echo "o  ${T_MD}conf/ssl.key/server.key${T_ME}"
@@ -155,7 +166,7 @@ case $type in
         echo ""
         echo "${T_MD}Generating test certificate signed by Snake Oil CA [TEST]${T_ME}"
         echo "WARNING: Do not use this for real-life/production systems"
-        if [ ".$algo" = . ]; then
+        if [ ".$algo" = .choose ]; then
             echo "______________________________________________________________________"
             echo ""
             echo "${T_MD}STEP 0: Decide the signature algorithm used for certificate${T_ME}"
@@ -186,7 +197,7 @@ case $type in
         if [ ! -f $HOME/.rnd ]; then
             touch $HOME/.rnd
         fi
-        if [ $algo = RSA ]; then
+        if [ ".$algo" = .RSA ]; then
             if [ ".$randfiles" != . ]; then
                 $openssl genrsa -rand $randfiles -out $sslkeydir/server.key 1024
             else
@@ -208,6 +219,7 @@ case $type in
                 exit 1
             fi
         fi
+        chmod 600 $sslkeydir/server.key
         echo "______________________________________________________________________"
         echo ""
         echo "${T_MD}STEP 2: Generating X.509 certificate signing request [server.csr]${T_ME}"
@@ -263,7 +275,7 @@ EOT
         if [ ! -f .mkcert.serial ]; then
             echo '01' >.mkcert.serial
         fi
-        if [ $algo = RSA ]; then
+        if [ ".$algo" = .RSA ]; then
             $openssl x509 $extfile \
                           -days 365 \
                           -CAserial .mkcert.serial \
@@ -287,7 +299,7 @@ EOT
         rm -f .mkcert.cfg
         echo "Verify: matching certificate & key modulus"
         modcrt=`$openssl x509 -noout -modulus -in $sslcrtdir/server.crt | sed -e 's;.*Modulus=;;'`
-        if [ $algo = RSA ]; then
+        if [ ".$algo" = .RSA ]; then
             modkey=`$openssl rsa -noout -modulus -in $sslkeydir/server.key | sed -e 's;.*Modulus=;;'`
         else
             modkey=`$openssl dsa -noout -modulus -in $sslkeydir/server.key | sed -e 's;.*Key=;;'`
@@ -297,7 +309,7 @@ EOT
             exit 1
         fi
         echo "Verify: matching certificate signature"
-        if [ $algo = RSA ]; then
+        if [ ".$algo" = .RSA ]; then
             $openssl verify -CAfile $sslcrtdir/snakeoil-ca-rsa.crt $sslcrtdir/server.crt
         else
             $openssl verify -CAfile $sslcrtdir/snakeoil-ca-dsa.crt $sslcrtdir/server.crt
@@ -325,7 +337,7 @@ EOT
             fi
         done
         if [ ".$rc" = .y ]; then
-            if [ $algo = RSA ]; then
+            if [ ".$algo" = .RSA ]; then
                 $openssl rsa -des3 \
                              -in  $sslkeydir/server.key \
                              -out $sslkeydir/server.key.crypt
@@ -340,6 +352,7 @@ EOT
             fi
             cp $sslkeydir/server.key.crypt $sslkeydir/server.key
             rm -f $sslkeydir/server.key.crypt
+            chmod 600 $sslkeydir/server.key
             echo "Fine, you're using an encrypted $algo private key."
         else
             echo "Warning, you're using an unencrypted $algo private key."
@@ -373,7 +386,7 @@ EOT
     custom)
         echo ""
         echo "${T_MD}Generating custom certificate signed by own CA [CUSTOM]${T_ME}"
-        if [ ".$algo" = . ]; then
+        if [ ".$algo" = .choose ]; then
             echo "______________________________________________________________________"
             echo ""
             echo "${T_MD}STEP 0: Decide the signature algorithm used for certificates${T_ME}"
@@ -404,7 +417,7 @@ EOT
         if [ ! -f $HOME/.rnd ]; then
             touch $HOME/.rnd
         fi
-        if [ $algo = RSA ]; then
+        if [ ".$algo" = .RSA ]; then
             if [ ".$randfiles" != . ]; then
                 $openssl genrsa -rand $randfiles -out $sslkeydir/ca.key 1024
             else
@@ -429,6 +442,7 @@ EOT
                 exit 1
             fi
         fi
+        chmod 600 $sslkeydir/ca.key
         echo "______________________________________________________________________"
         echo ""
         echo "${T_MD}STEP 2: Generating X.509 certificate signing request for CA [ca.csr]${T_ME}"
@@ -494,7 +508,7 @@ EOT
         rm -f .mkcert.cfg
         echo "Verify: matching certificate & key modulus"
         modcrt=`$openssl x509 -noout -modulus -in $sslcrtdir/ca.crt | sed -e 's;.*Modulus=;;'`
-        if [ $algo = RSA ]; then
+        if [ ".$algo" = .RSA ]; then
             modkey=`$openssl rsa -noout -modulus -in $sslkeydir/ca.key | sed -e 's;.*Modulus=;;'`
         else
             modkey=`$openssl dsa -noout -modulus -in $sslkeydir/ca.key | sed -e 's;.*Key=;;'`
@@ -515,7 +529,7 @@ EOT
         if [ ! -f $HOME/.rnd ]; then
             touch $HOME/.rnd
         fi
-        if [ $algo = RSA ]; then
+        if [ ".$algo" = .RSA ]; then
             if [ ".$randfiles" != . ]; then
                 $openssl genrsa -rand $randfiles -out $sslkeydir/server.key 1024
             else
@@ -536,6 +550,7 @@ EOT
                 exit 1
             fi
         fi
+        chmod 600 $sslkeydir/server.key
         echo "______________________________________________________________________"
         echo ""
         echo "${T_MD}STEP 5: Generating X.509 certificate signing request for SERVER [server.csr]${T_ME}"
@@ -605,7 +620,7 @@ EOT
         rm -f .mkcert.cfg
         echo "Verify: matching certificate & key modulus"
         modcrt=`$openssl x509 -noout -modulus -in $sslcrtdir/server.crt | sed -e 's;.*Modulus=;;'`
-        if [ $algo = RSA ]; then
+        if [ ".$algo" = .RSA ]; then
             modkey=`$openssl rsa -noout -modulus -in $sslkeydir/server.key | sed -e 's;.*Modulus=;;'`
         else
             modkey=`$openssl dsa -noout -modulus -in $sslkeydir/server.key | sed -e 's;.*Key=;;'`
@@ -639,7 +654,7 @@ EOT
             fi
         done
         if [ ".$rc" = .y ]; then
-            if [ $algo = RSA ]; then
+            if [ ".$algo" = .RSA ]; then
                 $openssl rsa -des3 \
                              -in  $sslkeydir/ca.key \
                              -out $sslkeydir/ca.key.crypt
@@ -654,6 +669,7 @@ EOT
             fi
             cp $sslkeydir/ca.key.crypt $sslkeydir/ca.key
             rm -f $sslkeydir/ca.key.crypt
+            chmod 600 $sslkeydir/ca.key
             echo "Fine, you're using an encrypted private key."
         else
             echo "Warning, you're using an unencrypted private key."
@@ -678,7 +694,7 @@ EOT
             fi
         done
         if [ ".$rc" = .y ]; then
-            if [ $algo = RSA ]; then
+            if [ ".$algo" = .RSA ]; then
                 $openssl rsa -des3 \
                              -in  $sslkeydir/server.key \
                              -out $sslkeydir/server.key.crypt
@@ -693,6 +709,7 @@ EOT
             fi
             cp $sslkeydir/server.key.crypt $sslkeydir/server.key
             rm -f $sslkeydir/server.key.crypt
+            chmod 600 $sslkeydir/server.key
             echo "Fine, you're using an encrypted $algo private key."
         else
             echo "Warning, you're using an unencrypted $algo private key."
@@ -760,6 +777,7 @@ EOT
             sed -e '/-----BEGIN ... PRIVATE KEY/,/-----END ... PRIVATE KEY/p' -e '/.*/d' \
                 <$key >$sslkeydir/server.key
         fi
+        chmod 600 $sslkeydir/server.key
         $openssl x509 -noout -in $sslcrtdir/server.crt
         if [ $? -ne 0 ]; then
             echo "mkcert.sh:Error: Failed to check certificate contents: $crt" 1>&2
